/* This file has the routines to save and restore the system variables from EEPROM.  Data that is saved:
    --The value for each field in the menu.  These values are stored in the first 20 locations.  One byte per value.
    --The HCI page number
    --The IR remote control key mapping.  Each IR action is allocated two bytes--the lower word of the IR value
    --ADAU1701 presets.  Format TBD
*/

#include <EEPROM.h>

const word state_start = 0;
const word IR_map_start = 32;
const word DSP_preset_start = 64;


void save_state() {
  byte count = 0;
  for (int i = 0; i < number_of_menu_pages; i++) {
    for (int j = 0; j < HCI.menupage[i].number_of_fields; j++) {
      EEPROM.update (state_start + count, HCI.menupage[i].page_field[j].field_value);
      count++;
    }
  }
  //save the current page
  EEPROM.update (state_start + count, current_page);

  //save the volume trims
  count++;
  EEPROM.update (state_start + count, Spkr1_offset);
  count++;
  EEPROM.update (state_start + count, Spkr2_offset);
  count++;
  EEPROM.update (state_start + count, Spkr3_offset);
  count++;
  EEPROM.update (state_start + count, Spkr4_offset);
}


void restore_state() {
  byte count = 0;
  byte temp;
  for (int i = 0; i < number_of_menu_pages; i++) {
    for (int j = 0; j < HCI.menupage[i].number_of_fields; j++) {
      temp = EEPROM.read (state_start + count);
      //check if value is blank:  replace with 0 if blank
      if (temp != 0xFF ) {
        HCI.menupage[i].page_field[j].field_value = temp;
        current_value(HCI.menupage[i].page_field[j].field_ID, HCI.menupage[i].page_field[j].field_value, false);  //update the hardware without changing the page
      }
      else {
        HCI.menupage[i].page_field[j].field_value = 0;
      }
      count++;
    }
  }

  //restore the current page, which is saved right after the field values:
  temp = EEPROM.read (state_start + count);
  //check if value is out of range:  if so, replace with 0
  if (temp <= HCI.number_of_pages) {
    current_page = temp;
  }
  else {
    current_page = 0;
  }

  //retore the volume trims
  count++;
  Spkr1_offset = EEPROM.read (state_start + count);
  count++;
  Spkr2_offset = EEPROM.read (state_start + count);
  count++;
  Spkr3_offset = EEPROM.read (state_start + count);
  count++;
  Spkr4_offset = EEPROM.read (state_start + count);
}


