/* This file contains all of the user input routines:  keyboard, rotary encoder, and IR control  */


/*This is the original keyboard routine for the buttons on the SainSmart LCD shield
   It calls the next_field and next_value routines to cursor through the menu
*/

void Port_init() {
  //initialize rotary encoder ports
  pinMode(PB_pin, INPUT_PULLUP);        //PB switch on rotary encoder
  pinMode(rot_enc2_pin, INPUT);       //Rotary encoder2
  pinMode(rot_enc1_pin, INPUT);       //Rotary encoder1
}

void KBD_check_buttons() {

  adc_key_in = analogRead(0);    // read the value from the sensor
  digitalWrite(13, HIGH);
  key = get_key(adc_key_in);            // convert into key press

  if (key != oldkey)            // if keypress is detected
  {
    delay(50);    // wait for debounce time
    adc_key_in = analogRead(0);    // read the value from the sensor
    key = get_key(adc_key_in);            // convert into key press
    if (key != oldkey)
    {
      oldkey = key;
      if (key >= 0) {
        switch (key) {
          case 4:       //"select" button
            next_field();
            break;
          case 1:       //"up key"
            next_value();
            break;
          case 2:       //"down" key
            previous_value();
            break;
        }
        //lcd.print(msgs[key]);
      }
    }
  }
}

//________________________________________________________________________________________________________
//
//  KBD Support Routine
//________________________________________________________________________________________________________
// Convert ADC value to key number
int get_key(unsigned int input)
{
  int k;

  for (k = 0; k < NUM_KEYS; k++)
  {
    if (input < adc_key_val[k])
    {

      return k;
    }
  }

  if (k >= NUM_KEYS)
    k = -1;     // No valid key pressed

  return k;
}

const byte pushed = 0;

void Rot_Enc_check() {
  /* This code checks the pushbutton.  It looks for a debounced "push" and then waits for a debounced release
     Waiting for the release allows implementing a "reverse" feature, where the HCI goes to the previous field if
     the pushbutton is held down for 1 second.  If the pushbutton is held for more than 8 seconds, the menu
     configuration screen will appear.  That menu will allow selecting which HCI screens are used for various boards */

  rot_enc_pb = digitalRead(PB_pin);
  if (rot_enc_pb == pushed)
  {
    delay(30);    // wait for debounce time
    rot_enc_pb = digitalRead(PB_pin);     //read key again after the debounce time
    if (rot_enc_pb == pushed) {             //key is down
      debounced_key = 0;
      long_count = millis();
      while (debounced_key == 0) {
        rot_enc_pb = digitalRead(PB_pin);   //read key again to see if released
        if (rot_enc_pb != pushed) {
          delay(50);
          rot_enc_pb = digitalRead(PB_pin);
          if (rot_enc_pb != pushed) {
            debounced_key = 1;
            if (millis() - long_count > 8000) {
              current_page = HCI.number_of_pages;  //go past "normal" pages to find the Remote Setup menu
              current_page_field = 0;
              HCI_page_refresh();
            }
            else if (millis() - long_count > 1000) previous_field();
            else next_field();
          }
        }
      }
    }
  }

  rot_enc_Q1 = (digitalRead(rot_enc1_pin) << 1) | digitalRead(rot_enc2_pin);    //read the encoder
  if (rot_enc_Q1 != rot_enc_Q_old) {                                            //if different from old value
    delay(1);
    rot_enc_Q2 = (digitalRead(rot_enc1_pin) << 1) | digitalRead(rot_enc2_pin);  //read it again
    if (rot_enc_Q1 == rot_enc_Q2) {                            //make sure the new value is seen twice in a row
      switch (rot_enc_Q_old) {
        case B00000000:           //these other cases are here in case an optical encoder is used
          if (rot_enc_Q1 == B00000001) previous_value();
          else if (rot_enc_Q1 == B00000010) next_value();
          break;
        case B00000001:
          if (rot_enc_Q1 == B00000011) previous_value();
          else if (rot_enc_Q1 == B00000000) next_value();
          break;
        case B00000010:
          if (rot_enc_Q1 == B00000000) previous_value();
          else if (rot_enc_Q1 == B00000011) next_value();
          break;
        case B00000011:                     //for this type of encoder, this should be the only case
          if (rot_enc_Q1 == B00000010) previous_value();
          else if (rot_enc_Q1 == B00000001) next_value();
          break;
      }
      //Now check for the detent, where code should be "3" for this type of rotary encoder
      //This code should be deleted if using an optical rotary encoder
      long_count = millis();
      while (rot_enc_Q1 != B00000011)  {      //look for two consecutive 3's
        rot_enc_Q1 = (digitalRead(rot_enc1_pin) << 1) | digitalRead(rot_enc2_pin);
        delay(1);
        rot_enc_Q2 = (digitalRead(rot_enc1_pin) << 1) | digitalRead(rot_enc2_pin);
        if (millis() - long_count > 1000) break;        //if encoder stuck between detents, exit...
        if (rot_enc_Q1 != rot_enc_Q2) rot_enc_Q1 = 0;   //if the two reads are different, keep looking
      }
      rot_enc_Q_old = rot_enc_Q1;
    }
  }
}


/*
    IR Remote Control Routines
*/
void IR_check() {
  int table_offset;
  int EEPROM_word;

  if (irrecv.decode(&results)) {

    //first check for a non-repeat key
    if (results.value != 0xFFFFFFFF) {
      //lcd.clear();
      //lcd.print(results.value,HEX);
      //see if this is an update to the remote control key map table.  If so, update the table and clear the flag
      if (IR_key_updatable) {
        table_offset = IR_map_start + 2 * IR_key_action;
        EEPROM.update(table_offset, highByte(results.value));
        EEPROM.update(table_offset + 1, lowByte(results.value));
        IR_key_updatable = false;
        delay(1000);
      }
      else {
        previous_IR_key = results.value;
        IR_in(results.value);
        delay(100);
      }
    }

    //process the repeat key
    else {
      IR_in(previous_IR_key);
      delay(10);
    }
    irrecv.resume(); // Receive the next value
  }

}


void IR_in(long IR_value) {
  //  Decode the key
  byte matched_key = 100;

  for (int i = 0; i < 10; i++) {
    if (EEPROM.read(IR_map_start + 2 * i) == highByte(IR_value)) {
      if (EEPROM.read(IR_map_start + 2 * i + 1) == lowByte(IR_value)) {
        matched_key = i;
        //lcd.print(i);
        break;
      }
    }
  }

  //remote key enumerations:  left, right, up, down, zero, one, two, three, four, mute
  enum matched_key {left, right, up, down, zero, one, two, three, four, mute };

  switch (matched_key) {
    case left:     //left
      previous_value();
      break;
    case right:     //right
      next_value();
      break;
    case up:     //up
      next_field();
      break;
    case down:     //down
      previous_field();
      break;
    case zero:     // 0
      current_value(spkr_sel, 0, true);
      break;
    case one:     // 1
      current_value(spkr_sel, 1, true);
      break;
    case two:     // 2
      current_value(spkr_sel, 2, true);
      break;
    case three:     // 3
      current_value(spkr_sel, 3, true);
      break;
    case four:     // 4
      current_value(spkr_sel, 4, true);
      break;
    case mute:
      if (muted) {
        unMute();
        HCI_page_refresh();
      }
      else {
        Mute();
        lcd.clear();
        lcd.print("     Muted");
      }
      break;
    default:
      break;
  }
}



