/*
 This code controls the PGA2311 preamp and relay switchboxes.  It runs on the Arduino 
 replacement CPU board for the popular PGA2311 preamp kit. 

 Much of the code is shared with the ADAU1701 controller.
*/

/*
 * This code is distributed under a Creative Commons Attribution-NonCommercial-ShareAlike license
 * See this URL for the legal code for this license:  https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode
 */


/*-----------------------------------------------------------------------------------------
 * External libraries 
 *----------------------------------------------------------------------------------------*/
#include <avr/pgmspace.h>
#include <LiquidCrystal.h>
#include <Wire.h>
#include <IRremote.h>


//Instantiate the LCD with the right pins
LiquidCrystal lcd(8,9,4,5,6,7); 

//Instantiate the IR receiver objects
int recv_pin = A2;
IRrecv irrecv(recv_pin);
decode_results results;


/*-----------------------------------------------------------------------------------------
 * Constants and definitions
 * 
 * Note:  the Arduino convention of "byte" causes warnings with the wire library, which 
 * expects to see uint8_8 rather than byte.  They are the same, but uint8_t is used to
 * minimize the number of warnings
 *----------------------------------------------------------------------------------------*/
 
const uint8_t cmd_buffer_length = 16;
const uint8_t ADAU1701_addr = 0x34;
const uint8_t PCA9534_addr = 0x20;
const uint8_t One = 1;
const uint8_t Two = 2;
const int PB_pin = 2;         //digital pin 4 used for the rotary encoder pushbutton
const int rot_enc1_pin = A1;  //rotary encoder 1
const int rot_enc2_pin = A0;  //rotary encoder 2
const int CS_pin = 3;
const int SDI_pin = 12;
const int SCLK_pin = 11;
const int Mute_pin = A3;
unsigned long long_count;
unsigned long previous_IR_key;


/*-----------------------------------------------------------------------------------------
 * Global Variables for Preamp and Relays
 *----------------------------------------------------------------------------------------*/ 
uint8_t PCA9534_output = 0;
uint8_t I2C_addr = PCA9534_addr;
byte Master_vol = 0;
byte Spkr1_offset = 12;
byte Spkr2_offset = 12;
byte Spkr3_offset = 12;
byte Spkr4_offset = 12;
byte Spkr_selected_offset = 12;
byte Right_offset = 0;
byte Left_offset = 0;
bool muted = false;



/*-----------------------------------------------------------------------------------------
 * Global Variables for HCI Routine
 *----------------------------------------------------------------------------------------*/ 
                   
int  adc_key_val[5] ={30, 150, 360, 535, 760 };
int NUM_KEYS = 5;
int adc_key_in;
int key=-1;
int oldkey=-1;
byte rot_enc_pb = -1;
byte rot_enc_pb_old = -1;
byte debounced_key = 0;
byte rot_enc_Q1  = 3;
byte rot_enc_Q2  = 3;
byte rot_enc_Q_old = 3;
bool IR_key_updatable = false;
byte IR_key_action = 0;


struct field {
  const char* caption_table_pointer;
  byte caption_length;
  byte caption_row;
  byte caption_column;
  byte number_of_values;
  const char* value_table_pointer;
  byte value_length;
  byte value_row;
  byte value_column;
  byte field_value;
  byte field_ID;
};

const byte number_of_menu_pages = 3;
struct page {
  byte number_of_fields;
  field page_field[2];
}; 

struct menu {
  byte number_of_pages;
  page menupage[number_of_menu_pages + 1];
};

byte current_page;
byte current_page_field;
byte current_field_ID;
  
static menu HCI;    //make an instance of the menu called HCI


void setup() {

  LCD_init ();            //set up the LCD
   
  Wire.begin();           // join I2C bus as the master
  
  Port_init();            //Initialize pins to read the dip switches and save the state
  PGA2311_init();         //Initialize the ports for the volume control

  Init_PCA9534();         //Configure the PCA9534 I/O port extender
   
  HCI_init ();            //set up the HCI table and initialize the display
  restore_state();        //load the saved values from EEPROM and update everything
  
  HCI_page_refresh();     //reload the saved page

  irrecv.enableIRIn();    // Start the IR receiver
  
}

/*
 *  The main loop monitors the serial port and SIP switch status. 
 */
 
void loop() {
  // main loop:  check for serial input and check for dipswitch changes

  //KBD_check_buttons();  //see Input.ino
  Rot_Enc_check();      //see Input.ino
  IR_check();           //see Input.ino

}






  




