/* This code controls the relays in the relay boxes for the A/B switcher and it also controls
    the relays on the PGA2311 board for the inputs.  Both relays are connected to the outputs
    of the PCA9534 I/O extender.  Since this chip is on the I2C bus, the Wire library is used.
*/

void select_spkr_relay(byte relay_number) {
  PCA9534_output = PCA9534_output & 0xf0;   //turn off lower 4 bits

  switch (relay_number) {
    case 0:
      //lower 4 bits already turned off -- just continue to I2C write
      break;
    case 1:
      PCA9534_output = PCA9534_output | 0x1;    //turn on D0
      break;
    case 2:
      PCA9534_output = PCA9534_output | 0x2;    //turn on D1
      break;
    case 3:
      PCA9534_output = PCA9534_output | 0x4;    //turn on D2
      break;
    case 4:
      PCA9534_output = PCA9534_output | 0x8;    //turn on D3
      break;
  }
  Update_PCA9534();
}


void select_vol_relay(byte relay_number) {
  PCA9534_output = PCA9534_output & 0x0f;   //turn off upper 4 bits

  switch (relay_number) {
    case 0:
      //lower 4 bits already turned off -- just continue to I2C write
      break;
    case 1:
      PCA9534_output = PCA9534_output | 0x20;    //turn on D5
      break;
    case 2:
      PCA9534_output = PCA9534_output | 0x40;    //turn on D6
      break;
    case 3:
      PCA9534_output = PCA9534_output | 0x80;    //turn on D7
      break;
  }
  Update_PCA9534();
}


void Init_PCA9534() {
  Wire.beginTransmission(PCA9534_addr);   // transmit to device at I2C_addr
  Wire.write(3);                          //command byte = config register
  Wire.write(0);                          //0's make all I/O pins outputs
  Wire.endTransmission();                 // stop transmitting

  Wire.beginTransmission(PCA9534_addr);   // transmit to device at I2C_addr
  Wire.write(1);                          //command byte = output port register
  Wire.write(0);                          //0's make all I/O pins low
  Wire.endTransmission();                 // stop transmitting
}

void Update_PCA9534() {
  Wire.beginTransmission(PCA9534_addr);   // transmit to device at I2C_addr
  Wire.write(1);                          //command byte = output port register
  Wire.write(PCA9534_output);             //turn on the right bits
  Wire.endTransmission();                 // stop transmitting
}



