/*  This routine "executes" the field selected by the HCI.  The input is the page number and field number.
     The routine looks up the field_ID and transfers control to the appropriate routine.
     This routine is called whenever the value of a field is updated and it is called on power-up to restore the hardware state
     It is also called by the number keys and Mute buttons on the IR Remote
*/

void update_from_HCI(byte i, byte j) {
  //save HCI state to EEPROM
  //save_state();     //this moved to the next value and previous value routines.  It doesn't save the state for number keys

  //Now determine what to do for each of the updated fields
  switch (HCI.menupage[i].page_field[j].field_ID) {

    case volume:
      Master_vol = (HCI.menupage[i].page_field[j].field_value) * 2 + PGA2311_offset;


      PGA2311_vol(Spkr_selected_offset);

      break;

    case input:
      switch (HCI.menupage[i].page_field[j].field_value) {
        case 0:
          select_vol_relay(1);
          break;
        case 1:
          select_vol_relay(2);
          break;
        case 2:
          select_vol_relay(3);
          break;
      }
      break;

    case spkr_sel:
      switch (HCI.menupage[i].page_field[j].field_value) {
        case 0:
          select_spkr_relay(0);
          Spkr_selected_offset = chan_trim_qty/2;
          HCI.menupage[i].page_field[1].field_value = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 1:
          select_spkr_relay(1);
          Spkr_selected_offset = Spkr1_offset;
          HCI.menupage[i].page_field[1].field_value = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 2:
          select_spkr_relay(2);
          Spkr_selected_offset = Spkr2_offset;
          HCI.menupage[i].page_field[1].field_value = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 3:
          select_spkr_relay(3);
          Spkr_selected_offset = Spkr3_offset;
          HCI.menupage[i].page_field[1].field_value = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 4:
          select_spkr_relay(4);
          Spkr_selected_offset = Spkr4_offset;
          HCI.menupage[i].page_field[1].field_value = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
      }
      break;

    case spkr_offset:
      //use the speaker selection to determine how to update the offsets
      switch (HCI.menupage[i].page_field[0].field_value) {
        case 1:
          Spkr_selected_offset = HCI.menupage[i].page_field[j].field_value;
          Spkr1_offset = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 2:
          Spkr_selected_offset = HCI.menupage[i].page_field[j].field_value;
          Spkr2_offset = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 3:
          Spkr_selected_offset = HCI.menupage[i].page_field[j].field_value;
          Spkr3_offset = Spkr_selected_offset;
          vol_and_HCI_update();
          break;
        case 4:
          Spkr_selected_offset = HCI.menupage[i].page_field[j].field_value;
          Spkr4_offset = Spkr_selected_offset;
          vol_and_HCI_update();
      }
      break;

    case IR_action:
      IR_key_action = HCI.menupage[i].page_field[j].field_value;
      break;

    case IR_Key_map:
      //Set a flag for the remote control routine to allow changing the existing value
      //Then allow IR routine to update the display.

      IR_key_updatable = true;    //tell the IR remote routine that the next value is to update the EEPROM IR key map
      break;
  }
}

void vol_and_HCI_update() {
  PGA2311_vol(Spkr_selected_offset);
  HCI_page_refresh();
}


